-- IBM_CUSTOMER_EVENT_PKG.PLS
-- Sample file for use with IBM WebSphere Adapter for Oracle E-Business Suite

-- Licensed Materials - Property of IBM
-- 5724N40
-- (C) Copyright IBM Corporation 2006 All Rights Reserved.
-- US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

CREATE OR REPLACE PACKAGE IBM_WEBSPHERE_PKG AS

  PROCEDURE raise_customer_event(
    errbuf                 OUT VARCHAR2,
    retcode                OUT NUMBER
  );

  FUNCTION rule_function(
    p_subscription IN            RAW,                                
    p_event        IN OUT NOCOPY WF_EVENT_T
  ) RETURN VARCHAR2;

  PROCEDURE generate_xml(
    p_customer_id IN  NUMBER,
    p_xml_doc     OUT CLOB
  );

  PROCEDURE is_customer_imported(
    p_itemtype   IN     VARCHAR2,
    p_itemkey    IN     VARCHAR2,
    p_actid      IN     NUMBER,
    p_funcmode   IN     VARCHAR2,
    p_x_result   IN OUT VARCHAR2
  );

END;
/

CREATE OR REPLACE PACKAGE BODY IBM_WEBSPHERE_PKG AS

  PROCEDURE raise_customer_event(
    errbuf                 OUT VARCHAR2,
    retcode                OUT NUMBER
  ) IS


    -- Cursor for fetching the changed/entered customers after the last completion date and time 
    -- of the IBM Raise Customer Event concurrent request
    CURSOR get_customer_ids(c_last_run_date VARCHAR2) IS
      SELECT cust_account_id,
             last_update_date,
             creation_date,
             status
      FROM   hz_cust_accounts hca
      WHERE  hca.last_update_date >= TO_DATE(c_last_run_date,'MM/DD/RRRR HH24:MI:SS');

    CURSOR get_last_run_date IS
      SELECT NVL(MAX(TO_CHAR(fcr.actual_completion_date,'MM/DD/RRRR HH24:MI:SS')),TO_CHAR(SYSDATE,'MM/DD/RRRR HH24:MI:SS'))
      FROM   fnd_concurrent_requests fcr,
             fnd_concurrent_programs fcp
      WHERE  fcr.concurrent_program_id = fcp.concurrent_program_id
      AND    fcp.concurrent_program_name = 'IBMCUSTOMEREVENT'
      AND    fcr.request_id NOT IN
             (SELECT MAX(request_id)
              FROM   fnd_concurrent_requests fcr,
                     fnd_concurrent_programs fcp
              WHERE  fcr.concurrent_program_id = fcp.concurrent_program_id
              AND    fcp.concurrent_program_name = 'IBMCUSTOMEREVENT'
             );

    v_last_run_date              VARCHAR2(21);
    v_customer_id                hz_cust_accounts.cust_account_id%TYPE;
    v_last_update_date           hz_cust_accounts.last_update_date%TYPE;
    v_creation_date              hz_cust_accounts.creation_date%TYPE;
    v_status                     hz_cust_accounts.status%TYPE;

    v_parameter_list             wf_parameter_list_t := wf_parameter_list_t(); 
    v_parameter_t                wf_parameter_t      := wf_parameter_t(NULL,NULL);
    v_event_key                  NUMBER;
    v_object_function            VARCHAR2(50);
    v_xml_data                   CLOB; 

  BEGIN
    -- Get the last request run date and time of the IBM Raise Customer Event concurrent request
    OPEN get_last_run_date;

    FETCH get_last_run_date
    INTO  v_last_run_date;

    IF get_last_run_date%ISOPEN THEN
      CLOSE get_last_run_date;
    END IF;

    fnd_file.put_line(FND_FILE.LOG, 'Last Concurrent Request Run Date and Time: '|| v_last_run_date);

    IF v_last_run_date IS NULL THEN
      v_last_run_date := TO_DATE(TRUNC(SYSDATE),'MM/DD/RRRR HH24:MI:SS');
    END IF;

    OPEN get_customer_ids(v_last_run_date);
    LOOP

      FETCH get_customer_ids
      INTO  v_customer_id,
            v_last_update_date,
            v_creation_date,
            v_status;

      EXIT WHEN get_customer_ids%NOTFOUND;

      fnd_file.put_line(FND_FILE.LOG, 'Raising event for customer id: '|| TO_CHAR(v_customer_id));

      SELECT ibm_customer_event_key_s.NEXTVAL                                           
      INTO   v_event_key                                                         
      FROM   dual;

      fnd_file.put_line(FND_FILE.LOG, 'Event Key: '|| TO_CHAR(v_event_key));                                                           

      IF v_creation_date = v_last_update_date THEN
        v_object_function := 'Create';
      ELSE
        v_object_function := 'Update';
      END IF;
      IF v_status <> 'A' THEN
        v_object_function := 'Delete';
      END IF;

      v_parameter_t.setname('#CONTEXT');                                      
      v_parameter_t.setvalue('CONCURRENT_PROGRAM');                                     
      v_parameter_list.extend;                                                   
      v_parameter_list(1) := v_parameter_t;
                                                                    
      v_parameter_t.setname('OBJECT_KEY');                                      
      v_parameter_t.setvalue(v_customer_id);                                     
      v_parameter_list.extend;                                                   
      v_parameter_list(2) := v_parameter_t;

      v_parameter_t.setname('OBJECT_NAME');                                      
      v_parameter_t.setvalue('ArIbm_R_Cust_AccountsBG');                                     
      v_parameter_list.extend;                                                   
      v_parameter_list(3) := v_parameter_t;

      v_parameter_t.setname('OBJECT_FUNCTION');                                      
      v_parameter_t.setvalue(v_object_function);                                     
      v_parameter_list.extend;                                                   
      v_parameter_list(4) := v_parameter_t;                                     

      v_parameter_t.setname('EVENT_PRIORITY');                                      
      v_parameter_t.setvalue(1);                                     
      v_parameter_list.extend;                                                   
      v_parameter_list(5) := v_parameter_t;

      v_parameter_t.setname('EVENT_TIME');
      v_parameter_t.setvalue(TO_CHAR(v_last_update_date,'MM/DD/RRRR HH24:MI:SS'));                                     
      v_parameter_list.extend;                                                   
      v_parameter_list(6) := v_parameter_t;

      v_parameter_t.setname('CONNECTOR_ID');
      v_parameter_t.setvalue('001');                                     
      v_parameter_list.extend;                                                   
      v_parameter_list(7) := v_parameter_t;
      
      generate_xml(v_customer_id,v_xml_data);
                                                                                
      wf_event.raise(                                                           
        p_event_name => 'ibm.apps.ar.customer.create',                 
        p_event_key  => v_event_key,                                     
        p_event_data => v_xml_data,                                           
        p_parameters => v_parameter_list                                
      );                                                              
                                                                                
      v_parameter_list.delete;

    END LOOP;

    IF get_customer_ids%ISOPEN THEN
      CLOSE get_customer_ids;
    END IF;

  END raise_customer_event;

  FUNCTION rule_function (
    p_subscription IN            RAW,                                
    p_event        IN OUT NOCOPY WF_EVENT_T
  ) RETURN VARCHAR2 IS                                                                           
                                                                                
   v_rule                      VARCHAR2(20);                                          
   v_parameter_list            wf_parameter_list_t      := wf_parameter_list_t();          
   v_parameter_t               wf_parameter_t           := wf_parameter_t(NULL, NULL);           
   v_parameter_name            v_parameter_t.name%TYPE;                               
   v_parameter_value           v_parameter_t.value%TYPE;                              
   v_parameter_count           PLS_INTEGER; 
   v_event_id                  NUMBER;   
   v_item_key                  VARCHAR2(2000);
   v_item_type                 VARCHAR2(2000);                                       
                                                                                
   v_object_key                v_parameter_t.value%TYPE;
   v_object_name               v_parameter_t.value%TYPE;
   v_object_function           v_parameter_t.value%TYPE;
   v_event_priority            v_parameter_t.value%TYPE;
   v_event_time                v_parameter_t.value%TYPE;
   v_context                   v_parameter_t.value%TYPE;  
   v_event_data                CLOB;                            
   v_connector_id              v_parameter_t.value%TYPE;                     
                                                                                
  BEGIN                                                                          

    v_parameter_list := p_event.getparameterlist(); 
    v_event_data := p_event.geteventdata();                            
  
    IF v_parameter_list IS NOT NULL THEN 

      v_parameter_count := v_parameter_list.FIRST;                                    

      WHILE( v_parameter_count <= v_parameter_list.LAST )                            
      LOOP 
        v_parameter_name  := v_parameter_list(v_parameter_count).getname();      
        v_parameter_value := v_parameter_list(v_parameter_count).getvalue();  

        IF v_parameter_name IS NOT NULL THEN                         
          IF v_parameter_name = '#CONTEXT' THEN
            BEGIN
              SELECT SUBSTR(v_parameter_value,1,INSTR(v_parameter_value,':',1)-1)
              INTO   v_context
              FROM   dual;
            EXCEPTION 
              WHEN OTHERS THEN
                v_context := 'CONCURRENT_PROGRAM';
            END;

            IF v_context IS NULL THEN
              v_context := 'CONCURRENT_PROGRAM';
            END IF; 

            BEGIN
              SELECT SUBSTR(v_parameter_value,INSTR(v_parameter_value,':',1)+1)
              INTO   v_item_key
              FROM   dual;
            EXCEPTION 
              WHEN OTHERS THEN
                v_item_key := NULL;
            END; 
 
          END IF;
        END IF;

        v_parameter_count := v_parameter_list.NEXT(v_parameter_count);

      END LOOP;

      v_item_type := v_context;

      IF v_context = 'CONCURRENT_PROGRAM' THEN                                      

        v_parameter_count := v_parameter_list.FIRST;                                    

        WHILE( v_parameter_count <= v_parameter_list.LAST )                            
        LOOP                                                            

          v_parameter_name  := NULL;                               
          v_parameter_value := NULL;                              
                                                                                 
          v_parameter_name  := v_parameter_list(v_parameter_count).getname();      
          v_parameter_value := v_parameter_list(v_parameter_count).getvalue();    
                                                                                
          IF v_parameter_name IS NOT NULL THEN                         
            IF v_parameter_name = 'OBJECT_KEY' THEN                                            
              v_object_key := v_parameter_value;     
            END IF;
            IF v_parameter_name = 'OBJECT_NAME' THEN                                            
              v_object_name := v_parameter_value;     
            END IF;
            IF v_parameter_name = 'OBJECT_FUNCTION' THEN                                            
              v_object_function := v_parameter_value;     
            END IF;
            IF v_parameter_name = 'EVENT_PRIORITY' THEN                                            
              v_event_priority := v_parameter_value;     
            END IF;
            IF v_parameter_name = 'EVENT_TIME' THEN                                            
              v_event_time := v_parameter_value;     
            END IF;                                         
            IF v_parameter_name = 'CONNECTOR_ID' THEN                                            
              v_connector_id := v_parameter_value;  
            END IF;                                     
          END IF;                                                 

          v_parameter_count := v_parameter_list.NEXT(v_parameter_count);                          

        END LOOP;

      ELSE

        v_object_key      := wf_engine.getitemattrtext(v_item_type,v_item_key,'OBJECT_KEY');
        v_object_name     := wf_engine.getitemattrtext(v_item_type,v_item_key,'OBJECT_NAME');
        v_object_function := wf_engine.getitemattrtext(v_item_type,v_item_key,'OBJECT_FUNCTION');
        v_event_priority  := wf_engine.getitemattrtext(v_item_type,v_item_key,'EVENT_PRIORITY');
        v_event_time      := TO_CHAR(wf_engine.getitemattrdate(v_item_type,v_item_key,'EVENT_TIME'),'MM/DD/RRRR HH24:MI:SS');
        v_connector_id    := wf_engine.getitemattrtext(v_item_type,v_item_key,'CONNECTOR_ID');

      END IF;

      SELECT ibm_websphere_events_s.NEXTVAL
      INTO   v_event_id
      FROM   DUAL;

      INSERT INTO ibm_websphere_events VALUES (
        v_event_id,                                                          --event id
        v_object_key,                                                        --object key
        v_object_name,                                                       --object name
        v_object_function,                                                   --object function
        v_event_priority,                                                    --event priority
        TO_DATE(v_event_time,'MM/DD/RRRR HH24:MI:SS'),                       --event time
        0,                                                                   --event status
        NULL,                                                                --event comment
        v_event_data,                                                        --object XML
        NULL,                                                                --xid value
        v_connector_id                                                       --connector id
      );

                                                                              
    END IF;                                                               
                                                                                
    v_rule :=  wf_rule.default_rule(p_subscription,p_event);               
                                                                                
    RETURN ('SUCCESS');                                                          
                                                                                
  END rule_function;

  PROCEDURE is_customer_imported(
    p_itemtype   IN     VARCHAR2,
    p_itemkey    IN     VARCHAR2,
    p_actid      IN     NUMBER,
    p_funcmode   IN     VARCHAR2,
    p_x_result   IN OUT VARCHAR2
  ) IS

  CURSOR get_count(c_customer_ref VARCHAR2) IS
    SELECT COUNT(*)
    FROM   ra_customers_interface_all
    WHERE  TRUNC(orig_system_customer_ref) = TRUNC(c_customer_ref);

  l_orig_sys_customer_ref     ra_customers_interface_all.orig_system_customer_ref%TYPE;
  l_error_count               NUMBER;

  BEGIN

    IF p_funcmode = 'RUN' THEN

      l_orig_sys_customer_ref := wf_engine.getitemattrtext( itemtype => p_itemtype,
                                               	      	    itemkey  => p_itemkey,
                                                      	    aname    => 'OBJECT_KEY'
						      	  );

      l_error_count := 0;

      OPEN get_count(l_orig_sys_customer_ref);

      FETCH get_count
      INTO  l_error_count;

      IF get_count%ISOPEN THEN
        CLOSE get_count;
      END IF;

      IF l_error_count = 0 THEN
        p_x_result := 'Y';
      ELSE
        p_x_result := 'N';
        wf_engine.setitemattrtext (itemtype   => p_itemtype,
                                   itemkey    => p_itemkey,
                                   aname      => 'OBJECT_FUNCTION',
                                   avalue     => 'Error');
      END IF;
    END IF;
    IF p_funcmode = 'CANCEL' THEN
      NULL;
      p_x_result := 'Y';
      RETURN;
    END IF;

  END is_customer_imported;

  PROCEDURE generate_xml(
    p_customer_id IN  NUMBER,
    p_xml_doc     OUT CLOB
    ) IS

    l_doc           xmldom.domdocument;
    l_main_node     xmldom.domnode;
    l_root_elmt     xmldom.domelement;
    l_root_node     xmldom.domnode;
    l_item_elmt     xmldom.domelement;
    l_user_node     xmldom.domnode;
    l_item_text     xmldom.domtext;
    l_customer_node xmldom.domnode;
    l_address_node  xmldom.domnode;
    l_add_det_node  xmldom.domnode;
    l_sub_node      xmldom.domnode;
    l_user_node1    xmldom.domnode;
    l_user_node2    xmldom.domnode;

    l_customer_name hz_parties.party_name%TYPE;
    l_customer_type hz_parties.party_type%TYPE;
    l_address1      hz_locations.address1%TYPE;
    l_address2      hz_locations.address2%TYPE;
    l_address3      hz_locations.address3%TYPE;
    l_city          hz_locations.city%TYPE;
    l_county        hz_locations.county%TYPE;
    l_country       hz_locations.country%TYPE;

    l_address_count NUMBER;

    CURSOR customer_details(c_customer_id NUMBER) IS
      SELECT party_name,
             party_type
      FROM   hz_cust_accounts hca,
             hz_parties hp
      WHERE  hca.cust_account_id = c_customer_id
      AND    hca.party_id = hp.party_id;

    CURSOR address_details(c_customer_id NUMBER) IS
      SELECT hl.address1,
             hl.address2,
             hl.address3,
             hl.city,
             hl.county,
             hl.country
      FROM   hz_cust_acct_sites_all hcasa,
             hz_party_sites hps,
             hz_locations hl
      WHERE  hcasa.cust_account_id = c_customer_id
      AND    hcasa.party_site_id = hps.party_site_id
      AND    hps.location_id = hl.location_id;

  BEGIN

    -- get document
    l_doc := xmldom.newdomdocument;
      
    -- create root element
    l_main_node := xmldom.makenode(l_doc);
    l_root_elmt := xmldom.createelement(l_doc,'CUSTOMER');

    -- set attribute to the root element
    xmldom.setattribute(l_root_elmt, 'xmlns:xsi', 'http://www.w3.org/2001/XMLSchema-instance');
    l_root_node := xmldom.appendchild(l_main_node, xmldom.makenode(l_root_elmt));

    -- get customer details
    OPEN customer_details(p_customer_id);
    LOOP
 
      FETCH customer_details
      INTO  l_customer_name,
            l_customer_type;

      EXIT WHEN customer_details%NOTFOUND;

      -- add customer node in the root node
      l_item_elmt := xmldom.createelement(l_doc,'CUSTOMER_DETAILS');
      l_customer_node := xmldom.appendchild(l_root_node, xmldom.makenode(l_item_elmt));

      -- add customer name node under customer node
      l_item_elmt := xmldom.createelement(l_doc,'CUSTOMER_NAME');
      l_user_node1 := xmldom.appendchild(l_customer_node, xmldom.makenode(l_item_elmt));
      IF l_customer_name IS NOT NULL THEN
        l_item_text := xmldom.createtextnode(l_doc,l_customer_name);
        l_user_node2 := xmldom.appendchild(l_user_node1, xmldom.makeNode(l_item_text));
      END IF;

      -- add customer type node under customer node
      l_item_elmt := xmldom.createelement(l_doc,'CUSTOMER_TYPE');
      l_user_node1 := xmldom.appendchild(l_customer_node, xmldom.makenode(l_item_elmt));
      IF l_customer_type IS NOT NULL THEN
        l_item_text := xmldom.createtextnode(l_doc,l_customer_type);
        l_user_node2 := xmldom.appendchild(l_user_node1, xmldom.makeNode(l_item_text));
      END IF;

      -- add customer address node under customer node
      l_item_elmt := xmldom.createelement(l_doc, 'CUSTOMER_ADDRESS');
      l_address_node := xmldom.appendchild(l_customer_node, xmldom.makeNode(l_item_elmt));
      l_address_count := 0;

      OPEN address_details(p_customer_id);
      LOOP

        FETCH address_details
        INTO  l_address1,
              l_address2,
              l_address3,
              l_city,
              l_county,
              l_country;

        EXIT WHEN address_details%NOTFOUND;

        -- set address attribute
        l_item_elmt := xmldom.createelement(l_doc, 'ADDRESS');
        l_add_det_node := xmldom.appendchild(l_address_node, xmldom.makenode(l_item_elmt));
        l_address_count := l_address_count+1;
        xmldom.setattribute(l_item_elmt, 'NUMBER',l_address_count);

        -- add address1 node under customer address node
        l_item_elmt := xmldom.createelement(l_doc,'ADDRESS1');
        l_user_node1 := xmldom.appendchild(l_add_det_node, xmldom.makenode(l_item_elmt));
        IF l_address1 IS NOT NULL THEN
          l_item_text := xmldom.createtextnode(l_doc,l_address1);
          l_user_node2 := xmldom.appendchild(l_user_node1, xmldom.makeNode(l_item_text));
        END IF;

        -- add address2 node under customer address node
        l_item_elmt := xmldom.createelement(l_doc,'ADDRESS2');
        l_user_node1 := xmldom.appendchild(l_add_det_node, xmldom.makenode(l_item_elmt));
        IF l_address2 IS NOT NULL THEN
          l_item_text := xmldom.createtextnode(l_doc,l_address2);
          l_user_node2 := xmldom.appendchild(l_user_node1, xmldom.makeNode(l_item_text));
        END IF;

        -- add address3 node under customer address node
        l_item_elmt := xmldom.createelement(l_doc,'ADDRESS3');
        l_user_node1 := xmldom.appendchild(l_add_det_node, xmldom.makenode(l_item_elmt));
        IF l_address3 IS NOT NULL THEN
          l_item_text := xmldom.createtextnode(l_doc,l_address3);
          l_user_node2 := xmldom.appendchild(l_user_node1, xmldom.makeNode(l_item_text));
        END IF;

        -- add city node under customer address node
        l_item_elmt := xmldom.createelement(l_doc,'CITY');
        l_user_node1 := xmldom.appendchild(l_add_det_node, xmldom.makenode(l_item_elmt));
        IF l_city IS NOT NULL THEN
          l_item_text := xmldom.createtextnode(l_doc,l_city);
          l_user_node2 := xmldom.appendchild(l_user_node1, xmldom.makeNode(l_item_text));
        END IF;

        -- add county node under customer address node
        l_item_elmt := xmldom.createelement(l_doc,'COUNTY');
        l_user_node1 := xmldom.appendchild(l_add_det_node, xmldom.makenode(l_item_elmt));
        IF l_county IS NOT NULL THEN
          l_item_text := xmldom.createtextnode(l_doc,l_county);
          l_user_node2 := xmldom.appendchild(l_user_node1, xmldom.makeNode(l_item_text));
        END IF;

        -- add country node under customer address node
        l_item_elmt := xmldom.createelement(l_doc,'COUNTRY');
        l_user_node1 := xmldom.appendchild(l_add_det_node, xmldom.makenode(l_item_elmt));
        IF l_country IS NOT NULL THEN
          l_item_text := xmldom.createtextnode(l_doc,l_country);
          l_user_node2 := xmldom.appendchild(l_user_node1, xmldom.makeNode(l_item_text));
        END IF;

      END LOOP;

      IF address_details%ISOPEN THEN
        CLOSE address_details;
      END IF;

    END LOOP;

    IF customer_details%ISOPEN THEN
      CLOSE customer_details;
    END IF;

    dbms_lob.createtemporary(p_xml_doc,false);
    xmldom.writetoclob(l_doc,p_xml_doc);
    xmldom.freedocument(l_doc);

  END generate_xml;

END IBM_WEBSPHERE_PKG;
/
SHOW ERRORS
